#ifndef _GLRC_STATSTICS_H_
#define _GLRC_STATSTICS_H_

#include "glrc.h"

#include <string>

namespace glrc
{

/**
 * Statistics rendering utility class.
 */
class GLRC_API Statistics
{
public:

    /**
     * Constructor. You can create an instance without valid OpenGL contexts.
     */
    Statistics();

    /**
     * Destructor.
     */
    virtual ~Statistics();

    /**
     * Initilaize OpenGL objects. Create bitmap font display lists.
     */
    virtual void init();

    /**
     * Show statistics.
     */
    void show();

    /**
     * Hide statistics.
     */
    void hide();

    /**
     * Check whether the statistics is shown or not.
     */
    bool isShown() const;

    /**
     * Set update interval.
     *
     * @param interval Interval count (1000 is default).
     */
    void setUpdateInterval(unsigned int interval);

    /**
     * Retrieve update interval.
     *
     * @return interval Interval count (1000 is default).
     */
    unsigned int getUpdateInterval() const;

    /**
     * Set additional description text.
     *
     * @param text Additional description text.
     */
    void setDescription(const char* text);

    /**
     * Retrieve the additional description text.
     *
     * @return Additional description text.
     */
    const char* getDescription() const;

    /**
     * Update FPS counting.
     */
    virtual void update();

    /**
     * Draw the statistics.
     */
    virtual void draw();

    /**
     * Set V-Sync waiting mode.
     *
     * @param V-Sync waiting mode.
     */
    void setWaitVSync(GLRC_VSYNC vsyncMode);

    /**
     * Retrieve V-Sync waiting mode.
     *
     * @return V-Sync waiting mode.
     */
    GLRC_VSYNC getWaitVSync() const;

protected:

    /**
     * Draw bitmap font string.
     *
     * @param x X (left) position.
     * @param y Y (top) position.
     * @param string String to draw.
     */
	void drawString(int x, int y, const char* string);

protected:

	unsigned int m_fontBase;            //!< Bitmap font display list index.

    bool m_bShow;                       //!< True if the statistics is shown.

    // for estimate FPS
#ifdef WIN32
	unsigned int m_timePeriodIndex;     //!< High resolution timer index.
#endif
    int m_frame;                        //!< Frame counter.
    int m_lastTime;                     //!< Time value at last update.
    float m_fps;                        //!< Current frame per second (FPS) value.
    unsigned int m_updateInterval;      //!< FPS interval count (default is 1000).

    std::string m_description;          //!< Additional description text.

    GLRC_VSYNC m_waitVSyncMode;         //!< Wait V-Sync mode.
};

}; // namespace glrc

#endif // _GLRC_STATISTICS_H_
