#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "glee.h"

#include <IL/il.h>
#include <IL/ilu.h>
#include <IL/ilut.h>

#include "glrc/glrc.h"
#include "glrc/Statistics.h"
extern "C" {
	#include "gltb.h"
}

#include <GL/glut.h>

unsigned int compositorIndex;
unsigned int particleTexIndex;
glrc::Statistics stats;

int wwidth;
int wheight;

/* Initialize */
static void init()
{
    /* initialize glrc lirbary */
	glrcInit();

    /* create a compositor */
	glrcGenCompositors(1, &compositorIndex);
	glrcBindCompositor(GLRC_COMPOSITOR_2D, compositorIndex);
    glrcSetupBuffer(GLRC_RTT_AUTO, 1024, 1024, TRUE, TRUE);

    /* setup effects */
	glrcSetDistortionMap("distort.png");
	glrcSetBlendMap("blend.png");
    glrcLoadMatrixFile("view.cfg");
	glrcSetExposure(0.1f);
    glrcDisable(GLRC_MESHDISTORTION);
	glrcSetEffectMode(GLRC_EFFECT_ALL);

    if (glIsTexture(particleTexIndex))
        glDeleteTextures(1, &particleTexIndex);
    particleTexIndex = ilutGLLoadImage("cloud.png");
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

	/* initialize trackball manipulator */
	gltbInit(GLUT_LEFT_BUTTON);
	gltbAnimate(GL_TRUE);

    stats.init();

	printf("GLRC HDR effect sample program\n");
	printf(" [Space] Enable or disable GLRC effects.\n");
	printf(" [1]     Use READPIXEL for off-screen rendering.\n");
	printf(" [2]     Use COPYSUBTEX for off-screen rendering.\n");
	printf(" [3]     Use PBUFFER for off-screen rendering.\n");
	printf(" [4]     Use PBUFFER_RTT for off-screen rendering.\n");
	printf(" [5]     Use FBO for off-screen rendering.\n");
	printf(" [f]     Enable or disable Floating point buffer.\n");
    printf(" [F1]    Change off-screen buffer size to 256 x 256.\n");
    printf(" [F2]    Change off-screen buffer size to 512 x 512.\n");
    printf(" [F3]    Change off-screen buffer size to 1024 x 1024.\n");
    printf(" [F4]    Change off-screen buffer size to 2048 x 2048.\n");
    printf(" [F9]    Enable or disable distortion correction.\n");
    printf(" [F10]   Enable or disable HDR rendering (need floating point buffer).\n");
    printf(" [F11]   Enable or disable color matrix conversion.\n");
    printf(" [Up]    Boost HDR exposure value.\n");
    printf(" [Down]  Reduce HDR exposure value.\n");
    printf(" [Left]  Boost HDR gamma value.\n");
    printf(" [Right] Reduce HDR gamma value.\n");
    printf(" [h]     Flip horizontally.\n");
    printf(" [v]     Flip vertically.\n");
    printf(" [s]     Show or hide statistics.\n");
	printf(" [w]     Enable or disable V-Sync waiting.\n");
	printf(" [o]     Take a snapshot in 2048x2048 size.\n");
	printf(" [ESC]   Exit this program.\n");

}

/* Draw a particles scene */
static void scene_particles(int numParticles)
{
	if (!glrcIsEnabled(GLRC_PROJECTION))
	{
		/* initialize projection matrix (without view matrix file) */
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(60.0, (double)wwidth / (double)wheight, 0.1, 100.0);
	}

    /* clear background */
    glClearColor(0.0, 0.0, 0.0, 1.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glDisable(GL_LIGHTING);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);
	glDisable(GL_DEPTH_TEST);

    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();

	if (!glrcIsEnabled(GLRC_MODELVIEW))
	{
		/* initialize model view matrix (without view matrix file) */
		glLoadIdentity();
	}

	/* place particles */
	glTranslated(0.0, 0.0, -3.0);

	/* apply trackball manipulator matrix */
	gltbMatrix();

    /* create particle data */
	static int bFirst = TRUE;
	static float* pos;
	if (bFirst)
	{
		pos = (float*)malloc(numParticles*3*sizeof(float));
		for (int i=0; i<numParticles; ++i) {
			pos[i*3+0] = (((float)rand()/RAND_MAX)-0.5f) * 2.0f;
			pos[i*3+1] = (((float)rand()/RAND_MAX)-0.5f) * 2.0f;
			pos[i*3+2] = (((float)rand()/RAND_MAX)-0.5f) * 2.0f;
		}
		bFirst = FALSE;
	}

	/* draw particles */
    glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	glPointSize(64.0f);
	glBindTexture(GL_TEXTURE_2D, particleTexIndex);
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	if (GL_ARB_point_sprite)
		glTexEnvf(GL_POINT_SPRITE_ARB, GL_COORD_REPLACE_ARB, GL_TRUE);
	glEnable(GL_TEXTURE_2D);
	if (GL_ARB_point_sprite)
		glEnable(GL_POINT_SPRITE_ARB);
	glBegin(GL_POINTS);
	for (int i=0; i<numParticles; ++i)
		glVertex3fv(&pos[i]);
	glEnd();
	if (GL_ARB_point_sprite)
		glDisable(GL_POINT_SPRITE_ARB);
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);

	glPopMatrix();
}

/* Display callback function */
static void display(void)
{
    /* activate compositor */
	glrcBindCompositor(GLRC_COMPOSITOR_2D, compositorIndex);

    /* clear background */
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    /* begin to render a scene into off-screen buffer */
	glrcBeginRender();

      /* draw the scene */
      scene_particles(5000);

    /* now use the off-screen rendered scene image to display */
	glrcEndRender();

    /* update and draw statistics */
    stats.update();
    stats.draw();

	glutSwapBuffers();
}

/* Resize callback function */
static void resize(int w, int h)
{
	wwidth = w;
	wheight = h;

	/* resize trackball input area */
	gltbReshape(w, h);

	/* setup viewport */
	glViewport(0, 0, w, h);
}

/* Mouse event callback function */
static void mouse(int button, int state, int x, int y)
{
	/* update trackball manipulator */
	gltbMouse(button, state, x, y);
}

/* Mouse drag callback function */
static void motion(int x, int y)
{
	/* update trackball manipulator */
	gltbMotion(x, y);
}

/* Keyboard input callback function */
static void keyboard(unsigned char key, int x, int y)
{
	switch (key)
	{
		case 'q':
		case 'Q':
		case '\033':
			/* ESC or 'q' key to exit */
			exit(0);
		case ' ':
            /* Space key to enable or disable GLRC effects */
			if (!glrcIsEnabled(GLRC_COMPOSITOR_2D))
				glrcEnable(GLRC_COMPOSITOR_2D);
			else
				glrcDisable(GLRC_COMPOSITOR_2D);
			glutPostRedisplay();
			break;
		case 'h':
            /* 'h' key to flip horizontally */
			glrcSetFlip(glrcGetFlip() ^ GLRC_FLIP_HORIZONTAL);
			glutPostRedisplay();
			break;
		case 'v':
            /* 'v' key to flip vertically */
			glrcSetFlip(glrcGetFlip() ^GLRC_FLIP_VERTICAL);
			glutPostRedisplay();
			break;
		case 's':
            /* 's' key to show or hide statistics */
			if (!stats.isShown())
				stats.show();
			else
				stats.hide();
			glutPostRedisplay();
			break;
		case 'w':
			/* 'w' key to enable or disable V-Sync waiting */
			if (stats.getWaitVSync() != GLRC_VSYNC_NOWAIT)
				stats.setWaitVSync(GLRC_VSYNC_NOWAIT);
			else
				stats.setWaitVSync(GLRC_VSYNC_WAIT);
			break;
		case 'f':
			/* 'f' key to enable or disable Floating point buffer */
			glrcSetupBuffer(glrcGetBufferType(), glrcGetBufferWidth(), glrcGetBufferHeight(), TRUE, !glrcIsFPBuffer());
			glutPostRedisplay();
			break;
		case '1':
			/* '1' key to use READPIXEL for off-screen rendering */
			glrcSetupBuffer(GLRC_RTT_READPIXEL, glrcGetBufferWidth(), glrcGetBufferHeight(), TRUE, glrcIsFPBuffer());
			glutPostRedisplay();
			break;
		case '2':
			/* '2' key to use COPYSUBTEX for off-screen rendering */
			glrcSetupBuffer(GLRC_RTT_COPYSUBTEX, glrcGetBufferWidth(), glrcGetBufferHeight(), TRUE, glrcIsFPBuffer());
			glutPostRedisplay();
			break;
		case '3':
			/* '3' key to use PBUFFER for off-screen rendering */
			glrcSetupBuffer(GLRC_RTT_PBUFFER, glrcGetBufferWidth(), glrcGetBufferHeight(), TRUE, glrcIsFPBuffer());
			glutPostRedisplay();
			break;
		case '4':
			/* '4' key to use PBUFFER_RTT for off-screen rendering */
			glrcSetupBuffer(GLRC_RTT_PBUFFER_RTT, glrcGetBufferWidth(), glrcGetBufferHeight(), TRUE, glrcIsFPBuffer());
			glutPostRedisplay();
			break;
		case '5':
			/* '5' key to use FBO for off-screen rendering */
			glrcSetupBuffer(GLRC_RTT_FBO, glrcGetBufferWidth(), glrcGetBufferHeight(), TRUE, glrcIsFPBuffer());
			glutPostRedisplay();
			break;
		case 'o':
			/* 'o' key to take a snapshot in 2048x2048 size */
			if (!glrcIsFPBuffer())
				glrcSaveSnapshot("output.png", 2048, 2048, FALSE, FALSE, display);
			else
				glrcSaveSnapshot("output.hdr", 2048, 2048, FALSE, TRUE, display);
			break;
		default:
			break;
	}
}

/* Special key input callback function */
static void specialKeyboard(int key, int x, int y)
{
	switch(key)
	{
		case GLUT_KEY_F1:
            /* 'F1' key to change off-screen buffer size to 256 x 256 */
			glrcSetupBuffer(glrcGetBufferType(), 256, 256, TRUE, glrcIsFPBuffer());
			glutPostRedisplay();
			break;
		case GLUT_KEY_F2:
            /* 'F2' key to change off-screen buffer size to 512 x 512 */
			glrcSetupBuffer(glrcGetBufferType(), 512, 512, TRUE, glrcIsFPBuffer());
			glutPostRedisplay();
			break;
		case GLUT_KEY_F3:
            /* 'F3' key to change off-screen buffer size to 1024 x 1024 */
			glrcSetupBuffer(glrcGetBufferType(), 1024, 1024, TRUE, glrcIsFPBuffer());
			glutPostRedisplay();
			break;
		case GLUT_KEY_F4:
            /* 'F4' key to change off-screen buffer size to 2048 x 2048 */
			glrcSetupBuffer(glrcGetBufferType(), 2048, 2048, TRUE, glrcIsFPBuffer());
			glutPostRedisplay();
			break;
		case GLUT_KEY_F9:
            /* 'F9' key to enable or disable distortion correction */
			glrcSetEffectMode(glrcGetEffectMode() ^ GLRC_EFFECT_DISTORTION);
			glutPostRedisplay();
			break;
		case GLUT_KEY_F10:
            /* 'F10' key to enable or disable high-dymanic range rendering (need floating point buffer) */
			glrcSetEffectMode(glrcGetEffectMode() ^ GLRC_EFFECT_HDR);
			glutPostRedisplay();
			break;
		case GLUT_KEY_F11:
            /* 'F11' key to enable or disable color matrix conversion */
			glrcSetEffectMode(glrcGetEffectMode() ^ GLRC_EFFECT_COLORMATRIX);
			glutPostRedisplay();
			break;
		case GLUT_KEY_UP:
            /* 'Up' cursor key to boost HDR exposure value */
			glrcSetExposure(glrcGetExposure() * 1.1f);
			glutPostRedisplay();
			break;
		case GLUT_KEY_DOWN:
            /* 'Down' cursor key to reduce HDR exposure value */
			glrcSetExposure(glrcGetExposure() * 0.9f);
			glutPostRedisplay();
			break;
		case GLUT_KEY_LEFT:
            /* 'Left' cursor key to boost HDR gamma value */
			glrcSetGamma(glrcGetGamma() - 0.1f);
			glutPostRedisplay();
			break;
		case GLUT_KEY_RIGHT:
            /* 'Right' cursor key to reduce HDR gamma value */
			glrcSetGamma(glrcGetGamma() + 0.1f);
			glutPostRedisplay();
			break;
	}
}

/* Main routine */
int main(int argc, char** argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGBA | GLUT_DEPTH | GLUT_DOUBLE);
	glutInitWindowSize(768, 768); 
	glutCreateWindow(argv[0]);
	glutDisplayFunc(display);
	glutReshapeFunc(resize);
	glutMouseFunc(mouse);
	glutMotionFunc(motion);
	glutKeyboardFunc(keyboard);
	glutSpecialFunc(specialKeyboard);

	init();
	
	glutMainLoop();

	glrcDestroy();

	return 0;
}
