#ifndef _SCENECONTENT_H_
#define _SCENECONTENT_H_

#include <QGLViewer/qglviewer.h>

namespace projection
{
class Scene;

/**
 * Scene content base class.
 */
class SceneContent
{
public:

    /**
     * Constructor.
     *
     * @param pScene Scene data.
     */
    SceneContent(Scene* pScene)
    {
        m_pScene = pScene;
    }

    /**
     * Destructor.
     */
    virtual ~SceneContent() {};

    /**
     * Retrieve screen content name.
     */
    QString virtual getName() const = 0;

    /**
     * Initialize screen content. (display list, etc.)
     */
    virtual void init() {};

    /**
     * Draw the scene content.
     *
     * @param cameraFrame View point. Useful for billboard  and so on.
     */
    virtual void draw(const qglviewer::Frame& cameraFrame) = 0;

    /**
     * Retrieve bounding box of the scene content.
     *
     * @param min One corner of the scene content.
     * @param max Another corner of the scene content.
     */
    virtual void getBoundingBox(qglviewer::Vec& min, qglviewer::Vec& max) = 0;

    /**
     * Restore the scene content from XML data.
     *
     * @param element Parent XML element of the scene content data.
     */
    virtual void initFromDOMElement(const QDomElement& element) = 0;

    /**
     * Store the current scene content as XML data.
     *
     * @param name XML node name of the data.
     * @param doc XML document to store the data.
     * @return Current scene content data as XML data.
     */
    virtual QDomElement domElement(const QString& name, QDomDocument& doc) const = 0;

protected:

    /**
     * Notify to redraw the scene content.
     */
    void notifyRedraw();

protected:

    Scene* m_pScene;    //!< Scene data.
};

}; // projection

#endif // _SCENECONTENT_H_
