#ifndef _SCENE_CUBEMAP_H_
#define _SCENE_CUBEMAP_H_

#include <string>

#include "Scene/SceneContent.h"

namespace projection
{

//! Cube map faces.
enum CUBEMAP_FACE
{
    CUBEMAP_FACE_FRONT=0,   //!< Front face.
    CUBEMAP_FACE_BACK,      //!< Back face.
    CUBEMAP_FACE_LEFT,      //!< Left face.
    CUBEMAP_FACE_RIGHT,     //!< Right face.
    CUBEMAP_FACE_TOP,       //!< Top face.
    CUBEMAP_FACE_BOTTOM     //!< Bottom face.
};

/**
 * Cube map scene class.
 */
class SceneCubeMap : public SceneContent
{
public:

    /**
     * Constructor.
     *
     * @param pScene Scene data.
     */
    SceneCubeMap(Scene* pScene);

    /**
     * Destructor.
     */
    virtual ~SceneCubeMap();

    /**
     * Retrieve screen content name.
     */
    QString getName() const { return "CubeMap"; }

    /**
     * Initialize screen content.
     */
    virtual void init();

    /**
     * Draw the scene content.
     *
     * @param cameraFrame View point.
     */
    virtual void draw(const qglviewer::Frame& cameraFrame);

    /**
     * Retrieve bounding box of the scene content.
     *
     * @param min One corner of the scene content.
     * @param max Another corner of the scene content.
     */
    void getBoundingBox(qglviewer::Vec& min, qglviewer::Vec& max);

    /**
     * Set texture to a face.
     *
     * @param face Face to set the texture.
     * @param fileName Texture file name.
     */
    void setTexture(CUBEMAP_FACE face, const QString& fileName);

    /**
     * Retrieve texture file name of the specified face.
     *
     * @param face Face to retrieve the texture file name
     * @return Texture file name.
     */
    QString getTexture(CUBEMAP_FACE face) const;

    /**
     * Restore the scene content from XML data.
     *
     * @param element Parent XML element of the scene content data.
     */
    virtual void initFromDOMElement(const QDomElement& element);

    /**
     * Store the current scene content as XML data.
     *
     * @param name XML node name of the data.
     * @param doc XML document to store the data.
     * @return Current scene content data as XML data.
     */
    virtual QDomElement domElement(const QString& name, QDomDocument& doc) const;

protected:

    QString m_textureFileNames[6];
    unsigned int m_textureIndex[6];
};

}; // projection

#endif // _SCENE_CUBEMAP_H_
