#ifndef _SCENE_EMPTY_H_
#define _SCENE_EMPTY_H_

#include "Scene/SceneContent.h"

namespace projection
{

/**
 * Empty scene class.
 */
class SceneEmpty : public SceneContent
{
public:

    /**
     * Constructor.
     *
     * @param pScene Scene data.
     */
    SceneEmpty(Scene* pScene);

    /**
     * Destructor.
     */
    virtual ~SceneEmpty();

    /**
     * Retrieve screen content name.
     */
    QString getName() const { return "Empty"; }

    /**
     * Initialize screen content. 
     */
    virtual void init();

    /**
     * Draw the scene content.
     *
     * @param cameraFrame View point.
     */
    virtual void draw(const qglviewer::Frame& cameraFrame);

    /**
     * Retrieve bounding box of the scene content.
     *
     * @param min One corner of the scene content.
     * @param max Another corner of the scene content.
     */
    void getBoundingBox(qglviewer::Vec& min, qglviewer::Vec& max);

    /**
     * Restore the scene content from XML data.
     *
     * @param element Parent XML element of the scene content data.
     */
    virtual void initFromDOMElement(const QDomElement& element);

    /**
     * Store the current scene content as XML data.
     *
     * @param name XML node name of the data.
     * @param doc XML document to store the data.
     * @return Current scene content data as XML data.
     */
    virtual QDomElement domElement(const QString& name, QDomDocument& doc) const;

protected:

};

}; // projection

#endif // _SCENE_EMPTY_H_
