#include "ProjectionModel.h"
#include "Channel.h"
#include "Warp.h"
#include "gui/QChannelWidget.h"

using namespace projection;

QChannelWidget::QChannelWidget(QWidget* pParent, Qt::WFlags flags)
    : QWidget(pParent, flags)
{
    ui.setupUi(this);

    ui.toolBox->setAutoFillBackground(false);

    setEnabled(false);

    m_pChannel = NULL;
}

QChannelWidget::~QChannelWidget()
{
}

void QChannelWidget::setChannel(Channel* pChannel)
{
    m_pChannel = pChannel;

    updateGUI();

    if (pChannel) {
        setEnabled(true);
    } else
        setEnabled(false);
}

Channel* QChannelWidget::getChannel() const
{
    return m_pChannel;
}

void QChannelWidget::updateGUI()
{
    if (m_pChannel)
    {
        ui.nameEdit->setText(m_pChannel->getName());
        ui.overlayNameCheckBox->setChecked(m_pChannel->getOverlayName());
        ui.remoteHostNameEdit->setText(m_pChannel->getRemoteHostName());
        ui.remoteFullScreenCheckBox->setChecked(m_pChannel->getRemoteFullScreen());
        ui.remoteScreenComboBox->setCurrentIndex(ui.remoteScreenComboBox->findText(QString::number(m_pChannel->getRemoteScreen())));
        ui.overlayImageEdit->setText(m_pChannel->getOverlayImageFileName());
        ui.projectorWidget->setFrustum(m_pChannel->getProjector());
        ui.projectorWidget->updateGUI();
        ui.warpCheckBox->setChecked(m_pChannel->getDistWarpEnabled());
        ui.warpResSpinBox->setValue(m_pChannel->getDistWarp()->getNumCtrlPoints());
        ui.viewWidget->setFrustum(m_pChannel->getView());
        ui.viewWidget->updateGUI();
    }
    else
    {
        ui.nameEdit->setText("");
        ui.overlayNameCheckBox->setChecked(false);
        ui.remoteHostNameEdit->setText("");
        ui.remoteFullScreenCheckBox->setChecked(false);
        ui.remoteScreenComboBox->setCurrentIndex(0);
        ui.overlayImageEdit->setText("");
        ui.projectorWidget->clear();
        ui.viewWidget->clear();
    }
}

void QChannelWidget::on_nameEdit_editingFinished()
{
    if (m_pChannel)
    {
        if (m_pChannel->getName() != ui.nameEdit->text())
        {
            m_pChannel->setName(ui.nameEdit->text());
            ui.nameEdit->setText(m_pChannel->getName());
        }
    }
}

void QChannelWidget::on_overlayNameCheckBox_toggled(bool checked)
{
    if (m_pChannel)
    {
        m_pChannel->setOverlayName(checked);
    }
}

void QChannelWidget::on_remoteHostNameEdit_editingFinished()
{
    if (m_pChannel)
    {
        if (m_pChannel->getRemoteHostName() != ui.remoteHostNameEdit->text())
            m_pChannel->setRemoteHostName(ui.remoteHostNameEdit->text());
    }
}

void QChannelWidget::on_remoteFullScreenCheckBox_toggled(bool checked)
{
    if (m_pChannel)
    {
        m_pChannel->setRemoteFullScreen(checked);
    }
}

void QChannelWidget::on_remoteScreenComboBox_activated(int)
{
    if (m_pChannel)
    {
        m_pChannel->setRemoteScreen(ui.remoteScreenComboBox->currentText().toInt());
    }
}

void QChannelWidget::on_overlayImageEdit_editingFinished()
{
    if (m_pChannel)
    {
        if (m_pChannel->getOverlayImageFileName() != ui.overlayImageEdit->text())
            m_pChannel->setOverlayImageFileName(ui.overlayImageEdit->text());
    }
}

void QChannelWidget::on_overlayImageBrowseButton_clicked()
{
    if (m_pChannel)
    {
        QString fileName = QFileDialog::getOpenFileName(this, "Choose a file to open", QString(), "Image files (*.*)");
        if (!fileName.isEmpty()) {
            ui.overlayImageEdit->setText(fileName);
            on_overlayImageEdit_editingFinished();
        }
    }
}

void QChannelWidget::on_warpCheckBox_toggled(bool checked)
{
    if (m_pChannel)
    {
        m_pChannel->setDistWarpEnabled(checked);
    }
}

void QChannelWidget::on_warpResSpinBox_valueChanged(int value)
{
    if (m_pChannel)
    {
        m_pChannel->getDistWarp()->setNumCtrlPoints(value);
        m_pChannel->updateData();
    }
}

void QChannelWidget::on_warpResetButton_clicked()
{
    if (m_pChannel)
    {
        m_pChannel->getDistWarp()->reset();
        m_pChannel->updateData();
    }
}

void QChannelWidget::on_calibrateButton_clicked()
{
}

void QChannelWidget::on_fitButton_clicked()
{
    if (m_pChannel)
    {
        m_pChannel->fitView();
    }
}

void QChannelWidget::on_windowButton_clicked()
{
    if (m_pChannel)
    {
        m_pChannel->showProjectorWindow();
    }
}
