#include "Frustum.h"
#include "gui/Defines.h"
#include "gui/QFrustumWidget.h"

using namespace projection;

QFrustumWidget::QFrustumWidget(QWidget* pParent, Qt::WFlags flags)
    : QWidget(pParent, flags)
{
    ui.setupUi(this);
    ui.transformWidget->setUseScale(false);

    m_pFrustum = NULL;
}

QFrustumWidget::~QFrustumWidget()
{
}

void QFrustumWidget::setFrustum(Frustum* pFrustum)
{
    m_pFrustum = pFrustum;

    ui.projectionWidget->disconnect();
    ui.transformWidget->disconnect();

    connect(ui.projectionWidget, SIGNAL(matrixChanged(const ProjectionMatrix&)),
            this, SLOT(on_projectionWidget_matrixChanged(const ProjectionMatrix&)));
	connect(ui.transformWidget, SIGNAL(matrixChanged(const TransformMatrix&)),
            this, SLOT(on_transformWidget_matrixChanged(const TransformMatrix&)));

	updateGUI();
}

void QFrustumWidget::clear()
{
    ui.projectionWidget->clear();
    ui.transformWidget->clear();
}

void QFrustumWidget::updateGUI()
{
    if (m_pFrustum)
    {
        ui.projectionWidget->setMatrix(m_pFrustum->getProjectionMatrix());
        ui.transformWidget->setMatrix(m_pFrustum->getTransformMatrix());
    }
}

void QFrustumWidget::on_projectionWidget_matrixChanged(const ProjectionMatrix& matrix)
{
    m_pFrustum->setProjectionMatrix(matrix);
}

void QFrustumWidget::on_transformWidget_matrixChanged(const TransformMatrix& matrix)
{
    m_pFrustum->setTransformMatrix(matrix);
}
