#include "ProjectionModel.h"
#include "Scene.h"
#include "Screen.h"
#include "RSync.h"
#include "Exporter.h"
#include "GUIControler.h"
#include "gui/Defines.h"
#include "gui/QDesignViewWindow.h"
#include "gui/QDesignViewWidget.h"
#include "gui/QPreferenceDialog.h"

using namespace projection;

QPreferenceDialog::QPreferenceDialog(QWidget* pParent, Qt::WFlags flags)
    : QDialog(pParent, flags)
{
    ui.setupUi(this);

    m_pModel = NULL;
}

QPreferenceDialog::~QPreferenceDialog()
{
}

void QPreferenceDialog::setModel(ProjectionModel* pModel)
{
    m_pModel = pModel;
    if (m_pModel->getRSync()->isServer())
        m_pDesignView = ((QDesignViewWindow*)m_pModel->getGUI()->getMainWindow())->getDesignView();

    updateGUI();
}

void QPreferenceDialog::updateGUI()
{
    if (m_pModel)
    {
        ui.gridSizeEdit->setText(formatValue(m_pDesignView->getGridFrameSize()));
        ui.gridSubdivSpinBox->setValue(m_pDesignView->getGridFrameSubdivisions());

        ui.bufferTypeComboBox->setCurrentIndex((int)m_pModel->getScreen()->getBufferType());
        ui.bufferWidthComboBox->setCurrentIndex(ui.bufferWidthComboBox->findText(QString::number(m_pModel->getScreen()->getBufferWidth())));
        ui.bufferHeightComboBox->setCurrentIndex(ui.bufferHeightComboBox->findText(QString::number(m_pModel->getScreen()->getBufferHeight())));

        ui.exportWidthComboBox->setCurrentIndex(ui.exportWidthComboBox->findText(QString::number(m_pModel->getExporter()->getExportWidth())));
        ui.exportHeightComboBox->setCurrentIndex(ui.exportHeightComboBox->findText(QString::number(m_pModel->getExporter()->getExportHeight())));
        ui.softEdgeCheckBox->setChecked(m_pModel->getExporter()->getSoftEdge());
        ui.blendEdgeWidthSpinBox->setValue(m_pModel->getExporter()->getBlendEdgeWidth());
        ui.blendEdgeExponentEdit->setText(QString::number(m_pModel->getExporter()->getBlendEdgeExponent()));
        ui.exportDistortMapPatternEdit->setText(m_pModel->getExporter()->getDistortMapFileNamePattern());
        ui.exportBlendMapPatternEdit->setText(m_pModel->getExporter()->getBlendMapFileNamePattern());
        ui.exportViewMatrixPatternEdit->setText(m_pModel->getExporter()->getViewMatrixFileNamePattern());
    }
}

void QPreferenceDialog::on_gridSizeEdit_editingFinished()
{
    if (m_pModel && m_pDesignView->getGridFrameSize() != ui.gridSizeEdit->text().toFloat())
		m_pDesignView->setGridFrameSize(ui.gridSizeEdit->text().toFloat());
}

void QPreferenceDialog::on_gridSubdivSpinBox_valueChanged(int value)
{
	if (m_pModel && m_pDesignView->getGridFrameSubdivisions() != value)
		m_pDesignView->setGridFrameSubdivisions(value);
}

void QPreferenceDialog::on_bufferTypeComboBox_activated(int)
{
    if (m_pModel)
        m_pModel->getScreen()->setBufferType((GLRC_RTT)ui.bufferTypeComboBox->currentIndex());
}

void QPreferenceDialog::on_bufferWidthComboBox_activated(int)
{
    if (m_pModel)
        m_pModel->getScreen()->setBufferSize(ui.bufferWidthComboBox->currentText().toInt(),
                                             ui.bufferHeightComboBox->currentText().toInt());
}

void QPreferenceDialog::on_bufferHeightComboBox_activated(int)
{
    if (m_pModel)
        m_pModel->getScreen()->setBufferSize(ui.bufferWidthComboBox->currentText().toInt(),
                                             ui.bufferHeightComboBox->currentText().toInt());
}

void QPreferenceDialog::on_exportWidthComboBox_activated(int)
{
	if (m_pModel)
		m_pModel->getExporter()->setExportWidth(ui.exportWidthComboBox->currentText().toInt());
}

void QPreferenceDialog::on_exportHeightComboBox_activated(int)
{
	if (m_pModel)
		m_pModel->getExporter()->setExportHeight(ui.exportHeightComboBox->currentText().toInt());
}

void QPreferenceDialog::on_softEdgeCheckBox_toggled(bool checked)
{
	if (m_pModel)
		m_pModel->getExporter()->setSoftEdge(checked);
}

void QPreferenceDialog::on_blendEdgeWidthSpinBox_valueChanged(int value)
{
	if (m_pModel)
		m_pModel->getExporter()->setBlendEdgeWidth(value);
}

void QPreferenceDialog::on_blendEdgeExponentEdit_editingFinished()
{
	if (m_pModel)
		m_pModel->getExporter()->setBlendEdgeExponent(ui.blendEdgeExponentEdit->text().toFloat());
}

void QPreferenceDialog::on_exportDistortMapPatternEdit_editingFinished()
{
    if (m_pModel)
        m_pModel->getExporter()->setDistortMapFileNamePattern(ui.exportDistortMapPatternEdit->text());
}

void QPreferenceDialog::on_exportBlendMapPatternEdit_editingFinished()
{
    if (m_pModel)
        m_pModel->getExporter()->setBlendMapFileNamePattern(ui.exportBlendMapPatternEdit->text());
}

void QPreferenceDialog::on_exportViewMatrixPatternEdit_editingFinished()
{
    if (m_pModel)
        m_pModel->getExporter()->setViewMatrixFileNamePattern(ui.exportViewMatrixPatternEdit->text());
}
