#include "ProjectionModel.h"
#include "Scene.h"
#include "gui/QSceneViewerWindow.h"

using namespace projection;

QSceneViewerWindow::QSceneViewerWindow(QWidget* pParent, const QGLWidget* pSharedWidget, Qt::WFlags flags) :
    QGLViewer(pParent, pSharedWidget, flags|Qt::Window)
{
    m_pModel = NULL;

	setWindowTitle("Scene Viewer");
	setFocusPolicy(Qt::StrongFocus);
    resize(640, 480);

	setShortcut(FULL_SCREEN, 0);
	setShortcut(EXIT_VIEWER, 0);
    setStateFileName(QString::null);

    // redrawrenderView
	connect(camera()->frame(), SIGNAL(modified()), this, SLOT(cameraChangedSlot()));
}

void QSceneViewerWindow::setModel(ProjectionModel* pModel)
{
    m_pModel = pModel;
	camera()->setPosition(m_pModel->getScene()->getCameraFrame().position());
	camera()->setOrientation(m_pModel->getScene()->getCameraFrame().orientation());

    connect(this, SIGNAL(cameraChanged(const qglviewer::Frame&)),
            m_pModel->getScene(), SLOT(updateCameraFrame(const qglviewer::Frame&)));
}

void QSceneViewerWindow::init()
{
}

void QSceneViewerWindow::draw()
{
    if (m_pModel)
    {
        m_pModel->getScene()->draw();
    }
}

void QSceneViewerWindow::resetCamera()
{
	qglviewer::Quaternion quat(qglviewer::Vec(1.0, 0.0, 0.0), 0.0/2.0);
	camera()->setOrientation(quat);
	camera()->setPosition(qglviewer::Vec(0.0f, -0.00001f, 0.0f));
    updateGL();
}

void QSceneViewerWindow::keyReleaseEvent(QKeyEvent* pEvent)
{
    QGLViewer::keyReleaseEvent(pEvent);

    static bool bFullScreen = false;
	switch (pEvent->key())
	{
	case Qt::Key_Return:
		if (pEvent->modifiers() & Qt::AltModifier)
		{
            if (!bFullScreen) {
				showFullScreen();
                bFullScreen = true;
            } else {
				showNormal();
                bFullScreen = false;
            }
		}
		break;
	default:
		return;
	}
}

void QSceneViewerWindow::cameraChangedSlot()
{
    emit cameraChanged(*(camera()->frame()));
}

void QSceneViewerWindow::initFromDOMElement(const QDomElement& element)
{
    if (element.isNull())
        return;

	QGLViewer::initFromDOMElement(element);

    if (element.attribute("show")=="true")
		show();
	else
		hide();
}

QDomElement QSceneViewerWindow::domElement(const QString& name, QDomDocument& doc) const
{
	QDomElement de = QGLViewer::domElement(name, doc);
	if (isVisible())
		de.setAttribute("show", "true");
	else
		de.setAttribute("show", "false");
    return de;
}
