#include "Scene.h"
#include "scene/SceneCubeMap.h"
#include "scene/SceneModel.h"
#include "ProjectionModel.h"
#include "gui/Defines.h"
#include "gui/QSceneWidget.h"

using namespace projection;

QSceneWidget::QSceneWidget(QWidget* pParent, Qt::WFlags flags)
    : QWidget(pParent, flags)
{
    ui.setupUi(this);

    m_pScene = NULL;
}

QSceneWidget::~QSceneWidget()
{
}

void QSceneWidget::setScene(Scene* pScene)
{
    m_pScene = pScene;
    ui.typeComboBox->clear();
    for (unsigned int i=0; i<m_pScene->getNumContents(); ++i)
        ui.typeComboBox->addItem(m_pScene->getContentName(i));

    updateGUI();
}

void QSceneWidget::updateGUI()
{
    if (m_pScene)
    {
        ui.transformWidget->setMatrix(m_pScene->getMatrix());
        if (m_pScene->getCurrentContentName() == "Empty")
        {
        }
        else if (m_pScene->getCurrentContentName() == "Teapot")
        {
        }
        else if (m_pScene->getCurrentContentName() == "CubeMap")
        {
            ui.frontFileNameEdit->setText(((SceneCubeMap*)m_pScene->getContent("CubeMap"))->getTexture(CUBEMAP_FACE_FRONT));
            ui.backFileNameEdit->setText(((SceneCubeMap*)m_pScene->getContent("CubeMap"))->getTexture(CUBEMAP_FACE_BACK));
            ui.leftFileNameEdit->setText(((SceneCubeMap*)m_pScene->getContent("CubeMap"))->getTexture(CUBEMAP_FACE_LEFT));
            ui.rightFileNameEdit->setText(((SceneCubeMap*)m_pScene->getContent("CubeMap"))->getTexture(CUBEMAP_FACE_RIGHT));
            ui.topFileNameEdit->setText(((SceneCubeMap*)m_pScene->getContent("CubeMap"))->getTexture(CUBEMAP_FACE_TOP));
            ui.bottomFileNameEdit->setText(((SceneCubeMap*)m_pScene->getContent("CubeMap"))->getTexture(CUBEMAP_FACE_BOTTOM));
        }
        else if (m_pScene->getCurrentContentName() == "Model")
        {
            ui.modelFileNameEdit->setText(((SceneModel*)m_pScene->getContent("Model"))->getFileName());
            ui.modelCullingCheckBox->setChecked(((SceneModel*)m_pScene->getContent("Model"))->getBackFaceCulling());
            ui.modelSmoothingCheckBox->setChecked(((SceneModel*)m_pScene->getContent("Model"))->getSmoothing());
        }
        ui.showInDesignViewCheckBox->setChecked(m_pScene->getShowInDesignView());
        ui.sceneSizeEdit->setText(formatValue(m_pScene->getSize()));
        ui.frameCheckBox->setChecked(m_pScene->getShowFrame());
        ui.gridCheckBox->setChecked(m_pScene->getShowGrid());
        ui.gridSizeEdit->setText(formatValue(m_pScene->getShowGrid()));

        ui.typeComboBox->setCurrentIndex(ui.typeComboBox->findText(m_pScene->getCurrentContentName()));
        ui.stackedWidget->setCurrentIndex(ui.typeComboBox->currentIndex());
    }
}

void QSceneWidget::on_transformWidget_matrixChanged(const TransformMatrix& matrix)
{
    if (m_pScene && m_pScene->getMatrix() != matrix)
    {
        m_pScene->setMatrix(matrix);
        updateGUI();
    }
}

void QSceneWidget::on_typeComboBox_activated(int)
{
    if (m_pScene)
    {
        m_pScene->setCurrentContent(ui.typeComboBox->currentText());
        updateGUI();
    }
}

void QSceneWidget::on_frontFileNameEdit_editingFinished()
{
    if (m_pScene)
        ((SceneCubeMap*)m_pScene->getContent("CubeMap"))->setTexture(CUBEMAP_FACE_FRONT, ui.frontFileNameEdit->text());
}

void QSceneWidget::on_frontBrowseButton_clicked()
{
    QString fileName = QFileDialog::getOpenFileName(this, "Choose a file to open", QString(), "Image files (*.*)");
    if (!fileName.isEmpty()) {
        ui.frontFileNameEdit->setText(fileName);
        on_frontFileNameEdit_editingFinished();
    }
}

void QSceneWidget::on_backFileNameEdit_editingFinished()
{
    if (m_pScene)
        ((SceneCubeMap*)m_pScene->getContent("CubeMap"))->setTexture(CUBEMAP_FACE_BACK, ui.backFileNameEdit->text());
}

void QSceneWidget::on_backBrowseButton_clicked()
{
    QString fileName = QFileDialog::getOpenFileName(this, "Choose a file to open", QString(), "Image files (*.*)");
    if (!fileName.isEmpty()) {
        ui.backFileNameEdit->setText(fileName);
        on_backFileNameEdit_editingFinished();
    }
}

void QSceneWidget::on_leftFileNameEdit_editingFinished()
{
    if (m_pScene)
        ((SceneCubeMap*)m_pScene->getContent("CubeMap"))->setTexture(CUBEMAP_FACE_LEFT, ui.leftFileNameEdit->text());
}

void QSceneWidget::on_leftBrowseButton_clicked()
{
    QString fileName = QFileDialog::getOpenFileName(this, "Choose a file to open", QString(), "Image files (*.*)");
    if (!fileName.isEmpty()) {
        ui.leftFileNameEdit->setText(fileName);
        on_leftFileNameEdit_editingFinished();
    }
}

void QSceneWidget::on_rightFileNameEdit_editingFinished()
{
    if (m_pScene)
        ((SceneCubeMap*)m_pScene->getContent("CubeMap"))->setTexture(CUBEMAP_FACE_RIGHT, ui.rightFileNameEdit->text());
}

void QSceneWidget::on_rightBrowseButton_clicked()
{
    QString fileName = QFileDialog::getOpenFileName(this, "Choose a file to open", QString(), "Image files (*.*)");
    if (!fileName.isEmpty()) {
        ui.rightFileNameEdit->setText(fileName);
        on_rightFileNameEdit_editingFinished();
    }
}

void QSceneWidget::on_topFileNameEdit_editingFinished()
{
    if (m_pScene)
        ((SceneCubeMap*)m_pScene->getContent("CubeMap"))->setTexture(CUBEMAP_FACE_TOP, ui.topFileNameEdit->text());
}

void QSceneWidget::on_topBrowseButton_clicked()
{
    QString fileName = QFileDialog::getOpenFileName(this, "Choose a file to open", QString(), "Image files (*.*)");
    if (!fileName.isEmpty()) {
        ui.topFileNameEdit->setText(fileName);
        on_topFileNameEdit_editingFinished();
    }
}

void QSceneWidget::on_bottomFileNameEdit_editingFinished()
{
    if (m_pScene)
        ((SceneCubeMap*)m_pScene->getContent("CubeMap"))->setTexture(CUBEMAP_FACE_BOTTOM, ui.bottomFileNameEdit->text());
}

void QSceneWidget::on_bottomBrowseButton_clicked()
{
    QString fileName = QFileDialog::getOpenFileName(this, "Choose a file to open", QString(), "Image files (*.*)");
    if (!fileName.isEmpty()) {
        ui.bottomFileNameEdit->setText(fileName);
        on_bottomFileNameEdit_editingFinished();
    }
}

void QSceneWidget::on_modelFileNameEdit_editingFinished()
{
    if (m_pScene)
        ((SceneModel*)m_pScene->getContent("Model"))->setFileName(ui.modelFileNameEdit->text());
}

void QSceneWidget::on_modelBrowseButton_clicked()
{
    QString fileName = QFileDialog::getOpenFileName(this, "Choose a file to open", QString(), "Model files (*.obj)");
    if (!fileName.isEmpty()) {
        ui.modelFileNameEdit->setText(fileName);
        on_modelFileNameEdit_editingFinished();
    }
}

void QSceneWidget::on_modelCullingCheckBox_toggled(bool checked)
{
    if (m_pScene)
        ((SceneModel*)m_pScene->getContent("Model"))->setBackFaceCulling(checked);
}

void QSceneWidget::on_modelSmoothingCheckBox_toggled(bool checked)
{
    if (m_pScene)
        ((SceneModel*)m_pScene->getContent("Model"))->setSmoothing(checked);
}

void QSceneWidget::on_showInDesignViewCheckBox_toggled(bool checked)
{
    if (m_pScene)
        m_pScene->setShowInDesignView(checked);
}

void QSceneWidget::on_sceneSizeEdit_editingFinished()
{
    if (m_pScene)
        m_pScene->setSize(formatValue(ui.sceneSizeEdit->text()));
}

void QSceneWidget::on_frameCheckBox_toggled(bool checked)
{
    if (m_pScene)
        m_pScene->setShowFrame(checked);
}

void QSceneWidget::on_gridCheckBox_toggled(bool checked)
{
    if (m_pScene)
        m_pScene->setShowGrid(checked);
}

void QSceneWidget::on_gridSizeEdit_editingFinished()
{
    if (m_pScene && m_pScene->getGridSize() != ui.gridSizeEdit->text().toDouble())
        m_pScene->setGridSize(ui.gridSizeEdit->text().toDouble());
}

void QSceneWidget::on_resetCameraButton_clicked()
{
    m_pScene->resetCamera();
}
